import { Subject } from 'rxjs';
import { EditViewBase } from '@/studio-core';
import HROMHierarchyCatService from '@/service/hromhierarchy-cat/hromhierarchy-cat-service';
import HROMHierarchyCatAuthService from '@/authservice/hromhierarchy-cat/hromhierarchy-cat-auth-service';
import EditViewEngine from '@engine/view/edit-view-engine';
import HROMHierarchyCatUIService from '@/uiservice/hromhierarchy-cat/hromhierarchy-cat-ui-service';

/**
 * 层次结构视图基类
 *
 * @export
 * @class HROMHierarchyCatV_002Base
 * @extends {EditViewBase}
 */
export class HROMHierarchyCatV_002Base extends EditViewBase {
    /**
     * 视图对应应用实体名称
     *
     * @protected
     * @type {string}
     * @memberof HROMHierarchyCatV_002Base
     */
    protected appDeName: string = 'hromhierarchycat';

    /**
     * 应用实体主键
     *
     * @protected
     * @type {string}
     * @memberof HROMHierarchyCatV_002Base
     */
    protected appDeKey: string = 'omhierarchycatid';

    /**
     * 应用实体主信息
     *
     * @protected
     * @type {string}
     * @memberof HROMHierarchyCatV_002Base
     */
    protected appDeMajor: string = 'omhierarchycatname';

    /**
     * 实体服务对象
     *
     * @type {HROMHierarchyCatService}
     * @memberof HROMHierarchyCatV_002Base
     */
    protected appEntityService: HROMHierarchyCatService = new HROMHierarchyCatService;

    /**
     * 实体权限服务对象
     *
     * @type HROMHierarchyCatUIService
     * @memberof HROMHierarchyCatV_002Base
     */
    public appUIService: HROMHierarchyCatUIService = new HROMHierarchyCatUIService(this.$store);


    /**
     * 计数器服务对象集合
     *
     * @protected
     * @type {Array<*>}
     * @memberof HROMHierarchyCatV_002Base
     */    
    protected counterServiceArray: Array<any> = [];

    /**
     * 视图模型数据
     *
     * @protected
     * @type {*}
     * @memberof HROMHierarchyCatV_002Base
     */
    protected model: any = {
        srfCaption: 'entities.hromhierarchycat.views.v_002.caption',
        srfTitle: 'entities.hromhierarchycat.views.v_002.title',
        srfSubTitle: 'entities.hromhierarchycat.views.v_002.subtitle',
        dataInfo: ''
    }

    /**
     * 容器模型
     *
     * @protected
     * @type {*}
     * @memberof HROMHierarchyCatV_002Base
     */
    protected containerModel: any = {
        view_form: { name: 'form', type: 'FORM' },
    };


	/**
     * 视图唯一标识
     *
     * @protected
     * @type {string}
     * @memberof ViewBase
     */
	protected viewtag: string = '1316a3add8f01af03edf978ef21af6b1';


    /**
     * 视图引擎
     *
     * @public
     * @type {Engine}
     * @memberof HROMHierarchyCatV_002Base
     */
    public engine: EditViewEngine = new EditViewEngine();

    /**
     * 引擎初始化
     *
     * @public
     * @memberof HROMHierarchyCatV_002Base
     */
    public engineInit(): void {
        this.engine.init({
            view: this,
            form: this.$refs.form,
            p2k: '0',
            keyPSDEField: 'hromhierarchycat',
            majorPSDEField: 'omhierarchycatname',
            isLoadDefault: true,
        });
    }

    /**
     * form 部件 save 事件
     *
     * @param {*} [args={}]
     * @param {*} $event
     * @memberof HROMHierarchyCatV_002Base
     */
    public form_save($event: any, $event2?: any): void {
        this.engine.onCtrlEvent('form', 'save', $event);
    }

    /**
     * form 部件 remove 事件
     *
     * @param {*} [args={}]
     * @param {*} $event
     * @memberof HROMHierarchyCatV_002Base
     */
    public form_remove($event: any, $event2?: any): void {
        this.engine.onCtrlEvent('form', 'remove', $event);
    }

    /**
     * form 部件 load 事件
     *
     * @param {*} [args={}]
     * @param {*} $event
     * @memberof HROMHierarchyCatV_002Base
     */
    public form_load($event: any, $event2?: any): void {
        this.engine.onCtrlEvent('form', 'load', $event);
    }


}