import { Subject } from 'rxjs';
import { OptionViewBase } from '@/studio-core';
import HRDutyCertService from '@/service/hrduty-cert/hrduty-cert-service';
import HRDutyCertAuthService from '@/authservice/hrduty-cert/hrduty-cert-auth-service';
import OptionViewEngine from '@engine/view/option-view-engine';
import HRDutyCertUIService from '@/uiservice/hrduty-cert/hrduty-cert-ui-service';

/**
 * 职务证书视图基类
 *
 * @export
 * @class HRDutyCertV_001Base
 * @extends {OptionViewBase}
 */
export class HRDutyCertV_001Base extends OptionViewBase {
    /**
     * 视图对应应用实体名称
     *
     * @protected
     * @type {string}
     * @memberof HRDutyCertV_001Base
     */
    protected appDeName: string = 'hrdutycert';

    /**
     * 应用实体主键
     *
     * @protected
     * @type {string}
     * @memberof HRDutyCertV_001Base
     */
    protected appDeKey: string = 'hrdutycertid';

    /**
     * 应用实体主信息
     *
     * @protected
     * @type {string}
     * @memberof HRDutyCertV_001Base
     */
    protected appDeMajor: string = 'hrdutycertname';

    /**
     * 实体服务对象
     *
     * @type {HRDutyCertService}
     * @memberof HRDutyCertV_001Base
     */
    protected appEntityService: HRDutyCertService = new HRDutyCertService;

    /**
     * 实体权限服务对象
     *
     * @type HRDutyCertUIService
     * @memberof HRDutyCertV_001Base
     */
    public appUIService: HRDutyCertUIService = new HRDutyCertUIService(this.$store);


    /**
     * 计数器服务对象集合
     *
     * @protected
     * @type {Array<*>}
     * @memberof HRDutyCertV_001Base
     */    
    protected counterServiceArray: Array<any> = [];

    /**
     * 视图模型数据
     *
     * @protected
     * @type {*}
     * @memberof HRDutyCertV_001Base
     */
    protected model: any = {
        srfCaption: 'entities.hrdutycert.views.v_001.caption',
        srfTitle: 'entities.hrdutycert.views.v_001.title',
        srfSubTitle: 'entities.hrdutycert.views.v_001.subtitle',
        dataInfo: ''
    }

    /**
     * 容器模型
     *
     * @protected
     * @type {*}
     * @memberof HRDutyCertV_001Base
     */
    protected containerModel: any = {
        view_form: { name: 'form', type: 'FORM' },
        view_okbtn: { name: 'okbtn', type: 'button', text: '确定', disabled: true },
        view_cancelbtn: { name: 'cancelbtn', type: 'button', text: '取消', disabled: false },
        view_leftbtn: { name: 'leftbtn', type: 'button', text: '左移', disabled: true },
        view_rightbtn: { name: 'rightbtn', type: 'button', text: '右移', disabled: true },
        view_allleftbtn: { name: 'allleftbtn', type: 'button', text: '全部左移', disabled: true },
        view_allrightbtn: { name: 'allrightbtn', type: 'button', text: '全部右移', disabled: true },
    };


	/**
     * 视图唯一标识
     *
     * @protected
     * @type {string}
     * @memberof ViewBase
     */
	protected viewtag: string = '2ee255b61cfa5e9bb8991f1421e807c3';


    /**
     * 视图引擎
     *
     * @public
     * @type {Engine}
     * @memberof HRDutyCertV_001Base
     */
    public engine: OptionViewEngine = new OptionViewEngine();

    /**
     * 引擎初始化
     *
     * @public
     * @memberof HRDutyCertV_001Base
     */
    public engineInit(): void {
        this.engine.init({
            view: this,
            form: this.$refs.form,
            p2k: '0',
            keyPSDEField: 'hrdutycert',
            majorPSDEField: 'hrdutycertname',
            isLoadDefault: true,
        });
    }

    /**
     * form 部件 save 事件
     *
     * @param {*} [args={}]
     * @param {*} $event
     * @memberof HRDutyCertV_001Base
     */
    public form_save($event: any, $event2?: any): void {
        this.engine.onCtrlEvent('form', 'save', $event);
    }

    /**
     * form 部件 remove 事件
     *
     * @param {*} [args={}]
     * @param {*} $event
     * @memberof HRDutyCertV_001Base
     */
    public form_remove($event: any, $event2?: any): void {
        this.engine.onCtrlEvent('form', 'remove', $event);
    }

    /**
     * form 部件 load 事件
     *
     * @param {*} [args={}]
     * @param {*} $event
     * @memberof HRDutyCertV_001Base
     */
    public form_load($event: any, $event2?: any): void {
        this.engine.onCtrlEvent('form', 'load', $event);
    }


}