import { Subject } from 'rxjs';
import { OptionViewBase } from '@/studio-core';
import HREmployeeService from '@/service/hremployee/hremployee-service';
import HREmployeeAuthService from '@/authservice/hremployee/hremployee-auth-service';
import OptionViewEngine from '@engine/view/option-view-engine';
import HREmployeeUIService from '@/uiservice/hremployee/hremployee-ui-service';

/**
 * 快速新建视图基类
 *
 * @export
 * @class HREmployeeV_001Base
 * @extends {OptionViewBase}
 */
export class HREmployeeV_001Base extends OptionViewBase {
    /**
     * 视图对应应用实体名称
     *
     * @protected
     * @type {string}
     * @memberof HREmployeeV_001Base
     */
    protected appDeName: string = 'hremployee';

    /**
     * 应用实体主键
     *
     * @protected
     * @type {string}
     * @memberof HREmployeeV_001Base
     */
    protected appDeKey: string = 'employeeid';

    /**
     * 应用实体主信息
     *
     * @protected
     * @type {string}
     * @memberof HREmployeeV_001Base
     */
    protected appDeMajor: string = 'employeename';

    /**
     * 实体服务对象
     *
     * @type {HREmployeeService}
     * @memberof HREmployeeV_001Base
     */
    protected appEntityService: HREmployeeService = new HREmployeeService;

    /**
     * 实体权限服务对象
     *
     * @type HREmployeeUIService
     * @memberof HREmployeeV_001Base
     */
    public appUIService: HREmployeeUIService = new HREmployeeUIService(this.$store);


    /**
     * 计数器服务对象集合
     *
     * @protected
     * @type {Array<*>}
     * @memberof HREmployeeV_001Base
     */    
    protected counterServiceArray: Array<any> = [];

    /**
     * 视图模型数据
     *
     * @protected
     * @type {*}
     * @memberof HREmployeeV_001Base
     */
    protected model: any = {
        srfCaption: 'entities.hremployee.views.v_001.caption',
        srfTitle: 'entities.hremployee.views.v_001.title',
        srfSubTitle: 'entities.hremployee.views.v_001.subtitle',
        dataInfo: ''
    }

    /**
     * 容器模型
     *
     * @protected
     * @type {*}
     * @memberof HREmployeeV_001Base
     */
    protected containerModel: any = {
        view_form: { name: 'form', type: 'FORM' },
        view_okbtn: { name: 'okbtn', type: 'button', text: '确定', disabled: true },
        view_cancelbtn: { name: 'cancelbtn', type: 'button', text: '取消', disabled: false },
        view_leftbtn: { name: 'leftbtn', type: 'button', text: '左移', disabled: true },
        view_rightbtn: { name: 'rightbtn', type: 'button', text: '右移', disabled: true },
        view_allleftbtn: { name: 'allleftbtn', type: 'button', text: '全部左移', disabled: true },
        view_allrightbtn: { name: 'allrightbtn', type: 'button', text: '全部右移', disabled: true },
    };


	/**
     * 视图唯一标识
     *
     * @protected
     * @type {string}
     * @memberof ViewBase
     */
	protected viewtag: string = '1fda62030d6149cb5ee6f088bc4f0aa7';


    /**
     * 视图引擎
     *
     * @public
     * @type {Engine}
     * @memberof HREmployeeV_001Base
     */
    public engine: OptionViewEngine = new OptionViewEngine();

    /**
     * 引擎初始化
     *
     * @public
     * @memberof HREmployeeV_001Base
     */
    public engineInit(): void {
        this.engine.init({
            view: this,
            form: this.$refs.form,
            p2k: '0',
            keyPSDEField: 'hremployee',
            majorPSDEField: 'employeename',
            isLoadDefault: true,
        });
    }

    /**
     * form 部件 save 事件
     *
     * @param {*} [args={}]
     * @param {*} $event
     * @memberof HREmployeeV_001Base
     */
    public form_save($event: any, $event2?: any): void {
        this.engine.onCtrlEvent('form', 'save', $event);
    }

    /**
     * form 部件 remove 事件
     *
     * @param {*} [args={}]
     * @param {*} $event
     * @memberof HREmployeeV_001Base
     */
    public form_remove($event: any, $event2?: any): void {
        this.engine.onCtrlEvent('form', 'remove', $event);
    }

    /**
     * form 部件 load 事件
     *
     * @param {*} [args={}]
     * @param {*} $event
     * @memberof HREmployeeV_001Base
     */
    public form_load($event: any, $event2?: any): void {
        this.engine.onCtrlEvent('form', 'load', $event);
    }


}