package cn.ibizlab.humanresource.core.humanresource.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.humanresource.domain.HRContract;
import cn.ibizlab.humanresource.core.humanresource.filter.HRContractSearchContext;
import cn.ibizlab.humanresource.core.humanresource.service.IHRContractService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.humanresource.mapper.HRContractMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[合同信息] 服务对象接口实现
 */
@Slf4j
@Service("HRContractServiceImpl")
public class HRContractServiceImpl extends ServiceImpl<HRContractMapper, HRContract> implements IHRContractService {

    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.humanresource.service.IHREmployeeService hremployeeService;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.base.service.IHROrganizationService hrorganizationService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HRContract et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getHrcontractid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HRContract> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HRContract et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("hrcontractid",et.getHrcontractid())))
            return false;
        CachedBeanCopier.copy(get(et.getHrcontractid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HRContract> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HRContract get(String key) {
        HRContract et = getById(key);
        if(et==null){
            et=new HRContract();
            et.setHrcontractid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HRContract getDraft(HRContract et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(HRContract et) {
        return (!ObjectUtils.isEmpty(et.getHrcontractid()))&&(!Objects.isNull(this.getById(et.getHrcontractid())));
    }
    @Override
    @Transactional
    public boolean save(HRContract et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HRContract et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HRContract> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HRContract> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<HRContract> selectByHremployeeid(String employeeid) {
        return baseMapper.selectByHremployeeid(employeeid);
    }

    @Override
    public void removeByHremployeeid(String employeeid) {
        this.remove(new QueryWrapper<HRContract>().eq("hremployeeid",employeeid));
    }

	@Override
    public List<HRContract> selectByMgrorganizationid(String organizationid) {
        return baseMapper.selectByMgrorganizationid(organizationid);
    }

    @Override
    public void removeByMgrorganizationid(String organizationid) {
        this.remove(new QueryWrapper<HRContract>().eq("mgrorganizationid",organizationid));
    }

	@Override
    public List<HRContract> selectBySignorganizationid(String organizationid) {
        return baseMapper.selectBySignorganizationid(organizationid);
    }

    @Override
    public void removeBySignorganizationid(String organizationid) {
        this.remove(new QueryWrapper<HRContract>().eq("signorganizationid",organizationid));
    }


    /**
     * 查询集合 执行中
     */
    @Override
    public Page<HRContract> searchActive(HRContractSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HRContract> pages=baseMapper.searchActive(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HRContract>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HRContract> searchDefault(HRContractSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HRContract> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HRContract>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 待续签
     */
    @Override
    public Page<HRContract> searchWaitSign(HRContractSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HRContract> pages=baseMapper.searchWaitSign(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HRContract>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 待终止
     */
    @Override
    public Page<HRContract> searchWaitStop(HRContractSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HRContract> pages=baseMapper.searchWaitStop(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HRContract>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(HRContract et){
        //实体关系[DER1N_HRCONTRACT_HREMPLOYEE_HREMPLOYEEID_327A4D]
        if(!ObjectUtils.isEmpty(et.getHremployeeid())){
            cn.ibizlab.humanresource.core.humanresource.domain.HREmployee hremployee=et.getHremployee();
            if(ObjectUtils.isEmpty(hremployee)){
                cn.ibizlab.humanresource.core.humanresource.domain.HREmployee majorEntity=hremployeeService.get(et.getHremployeeid());
                et.setHremployee(majorEntity);
                hremployee=majorEntity;
            }
            et.setHremployeename(hremployee.getEmployeename());
            et.setOrganizationname(hremployee.getOrganizationname());
            et.setEmpstate(hremployee.getEmpstate());
            et.setStartorgtime(hremployee.getStartorgtime());
            et.setEmployeecode(hremployee.getEmployeecode());
        }
        //实体关系[DER1N_HRCONTRACT_HRORGANIZATION_MGRORGANIZATIONID_6BD9DC]
        if(!ObjectUtils.isEmpty(et.getMgrorganizationid())){
            cn.ibizlab.humanresource.core.base.domain.HROrganization mgrorganization=et.getMgrorganization();
            if(ObjectUtils.isEmpty(mgrorganization)){
                cn.ibizlab.humanresource.core.base.domain.HROrganization majorEntity=hrorganizationService.get(et.getMgrorganizationid());
                et.setMgrorganization(majorEntity);
                mgrorganization=majorEntity;
            }
            et.setMgrorganizationname(mgrorganization.getOrganizationname());
        }
        //实体关系[DER1N_HRCONTRACT_HRORGANIZATION_SIGNORGANIZATIONID_6BD9DC]
        if(!ObjectUtils.isEmpty(et.getSignorganizationid())){
            cn.ibizlab.humanresource.core.base.domain.HROrganization signorganization=et.getSignorganization();
            if(ObjectUtils.isEmpty(signorganization)){
                cn.ibizlab.humanresource.core.base.domain.HROrganization majorEntity=hrorganizationService.get(et.getSignorganizationid());
                et.setSignorganization(majorEntity);
                signorganization=majorEntity;
            }
            et.setSignorganizationname(signorganization.getOrganizationname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HRContract> getHrcontractByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HRContract> getHrcontractByEntities(List<HRContract> entities) {
        List ids =new ArrayList();
        for(HRContract entity : entities){
            Serializable id=entity.getHrcontractid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



